		list p=16f84
		include p16f84.inc
 __CONFIG (_XT_OSC & _WDT_OFF & _PWRTE_ON & _CP_OFF)
 radix dec


; PORTB bits
#define CARD_PRESENT PORTB,4 ; input
#define DATA         PORTB,3 ; input
#define CLOCK        PORTB,2 ; input
#define SER_OUT      PORTB,1 ; output


 cblock 0x0c		; start of data memory
	serial_char
	parity_count
	byte_count
	bit_count
	card_data:40
 endc

; Serial timings
;  4Mhz = 1000ns/instruction
#define SER_2400 d'134'
#define SER_9600 d'31'	; untested

#define SER_CONSTANT SER_2400


 org 0x0000
	goto init				; reset vector address
	nop
	nop
	nop
	nop

init
	bcf STATUS,RP0
	bcf STATUS,RP1			; go to bank 0
	bsf SER_OUT				; Preset the serial output latch to off

	movlw b'00011101'		; Port B configuration byte.
	bsf STATUS,RP0			; change to register bank 1
	movwf TRISB				; setup port B
	bcf STATUS,RP0			; back to register bank 0

wait_for_card
	btfsc CARD_PRESENT		; wait for a card to be swiped
	goto wait_for_card

	clrf bit_count
find_start_sentinel			; read bit stream until SS (11010) is found
	btfsc CARD_PRESENT		; error if card isn't there anymore
	goto card_read_error
	btfsc CLOCK				; wait for clock pulse
	goto find_start_sentinel
	bcf STATUS,C			; clear carry flag
	rlf bit_count,0			; w=bit_count<<1
	movwf parity_count
	rlf parity_count,0		; w=bit_count<<2
	addwf PCL,F				; do a code table jump

	btfss DATA				; if DATA=1
	goto _bit_is_good		;   then do next bit
	clrf bit_count			; else clear bit_count
	goto _bit_is_not_good

	btfss DATA				; if DATA=1
	goto _bit_is_good		;   then do next bit
	clrf bit_count			; else clear bit_count
	goto _bit_is_not_good

	btfsc DATA				; if DATA=0
	goto _bit_is_good		;   then do next bit
	clrf bit_count			; else clear bit_count
	goto _bit_is_not_good

	btfss DATA				; if DATA=1
	goto _bit_is_good		;   then do next bit
	clrf bit_count			; else clear bit_count
	goto _bit_is_not_good

	btfsc DATA				; if DATA=0
	goto _bit_is_good		;   then do next bit
	clrf bit_count			; else clear bit_count
	goto _bit_is_not_good

	goto read_data

_bit_is_good
	incf bit_count,F
_bit_is_not_good
	btfss CLOCK				; wait for end of clock pulse
	goto _bit_is_not_good
	goto find_start_sentinel

read_data
	movlw card_data
	movwf FSR				; pointer to 1st block of card_data
	clrf byte_count
read_bytes_loop
	clrf INDF				; clear new data location
	clrf parity_count		; clear parity count
	movlw 5
	movwf bit_count			; read 5 bits/byte (4 data + parity)
read_bits_loop
	btfsc CARD_PRESENT
	goto send_data
	btfss CLOCK
	goto get_data_bit
	goto read_bits_loop
get_data_bit
	bcf STATUS,C			; clear carry flag
	btfsc DATA
	goto data_bit_in_c
	bsf STATUS,C			; get data bit into carry flag
	incf parity_count,F		; bit is 1, increase parity count
data_bit_in_c
	btfss CLOCK				; wait for end of clock pulse
	goto data_bit_in_c
	decfsz bit_count,F
	goto save_bit
	goto check_parity
save_bit
	rrf INDF,F				; rotate bit into final byte
	goto read_bits_loop
check_parity
	btfss parity_count,0	; check for odd parity
	goto card_read_error
	swapf INDF,F			; swap nibbles in card data byte
	incf byte_count,F
	movlw h'f1'
	addwf INDF,0			; w=INDF+F1h
	btfsc STATUS,Z
	goto send_data
	incf FSR,F
	goto read_bytes_loop

send_data
	movlw card_data
	movwf FSR				; set pointer to card_data
	movlw '+'
	movwf serial_char
	call rs232_send			; send '+' to indicate success
send_data_loop
	movf INDF,0
	addlw h'30'
	movwf serial_char		; put data+30h into serial_char
	call rs232_send
	incf FSR,F
	decfsz byte_count,F
	goto send_data_loop
wait_end_card
	btfss CARD_PRESENT
	goto wait_end_card
	goto wait_for_card

card_read_error
	btfss CARD_PRESENT
	goto card_read_error
	movlw '-'
	movwf serial_char
	call rs232_send
	goto wait_for_card

rs232_send
	bcf SER_OUT				; set start bit (start = 0)
	call serial_delay
	movlw 9
	movwf bit_count			; 9 more bits to go (8 data + stop bit)
	bsf STATUS,C			; set carry flag (stop bit = 1)
rs232_send_loop
	rrf serial_char,F
	btfsc STATUS,C
	bsf SER_OUT
	btfss STATUS,C
	bcf SER_OUT
	call serial_delay
	decfsz bit_count,F
	goto rs232_send_loop
	return

serial_delay
	movlw SER_CONSTANT
	movwf parity_count
serial_delay_loop
	decfsz parity_count,F
	goto serial_delay_loop
	end
